package org.mydomain.hellosmack;

import java.util.Collection;
import java.util.Set;

import org.jivesoftware.smack.SmackConfiguration;
import org.jivesoftware.smack.AbstractXMPPConnection;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.tcp.*;

import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.Message;

import org.jivesoftware.smack.chat2.Chat;
import org.jivesoftware.smack.chat2.ChatManager;
import org.jivesoftware.smack.chat2.IncomingChatMessageListener;

import org.jivesoftware.smackx.muc.MultiUserChatManager;
import org.jivesoftware.smackx.muc.MultiUserChat;


//import org.jivesoftware.smack.chat2.GroupChat;
//import org.jivesoftware.smack.im.*;
import org.jivesoftware.smack.roster.*;

import org.jxmpp.jid.Jid;
import org.jxmpp.jid.EntityBareJid;
import org.jxmpp.jid.impl.JidCreate;
import org.jxmpp.jid.parts.Resourcepart;
import org.jxmpp.jid.util.JidUtil;

// reference: https://github.com/igniterealtime/Smack/blob/master/documentation/gettingstarted.md

/*
 * Smack provides two ways to read incoming packets: StanzaListener, and StanzaCollector. 
 * Both use StanzaFilter instances to determine which stanzas should be processed. 
 * A stanza listener is used for event style programming, while a stanza collector has a result queue 
 * of packets that you can do polling and blocking operations on. 
 * So, a stanza listener is useful when you want to take some action whenever a stanza happens to come in, 
 * while a stanza collector is useful when you want to wait for a specific packet to arrive. 
 * Stanza collectors and listeners can be created using an Connection instance.
 */

public class HelloSmack
{

    // presenceType can be: available, error, probe, suscribe, subscribed, unavailable, unsubscribe, unsubscribed
    // reference: http://download.igniterealtime.org/smack/docs/latest/javadoc/org/jivesoftware/smack/packet/Presence.Type.html

    // TO BE TESTED
    private static void setPresence(XMPPConnection connection, Presence.Type presenceType /* e.g. Presence.Type.unavailable */, String message /* e.g. "Gone fishing" */ )
        throws org.jivesoftware.smack.SmackException.NotConnectedException,
               InterruptedException {
        Presence presence = new Presence(presenceType);
        presence.setStatus(message);
        connection.sendStanza(presence);
    }

    // TO BE TESTED
    private void printRosterEntries(XMPPConnection connection) {
        Roster roster = Roster.getInstanceFor(connection);
        Collection<RosterEntry> entries = roster.getEntries();
        for (RosterEntry entry : entries) {
            // each entry consists of: a JID (Jabber ID, i.e. an XMPP address), a user's name, and the list of all the entry's groups;
            // an entry with no group is an "unfiled entry"
            System.out.println(entry);
            // one can get the Presence status of an entry with:
            // Roster.getPresence(BareJid user)
            // --> will return null if the user is offline or if the caller is not subscribed to the user's presence
            // IMPORTANT: the presence subscription is not related to the membership in another user's roster
        }
    }

    // TO BE TESTED
    private void initializeRoster(XMPPConnection connection) {
        // example from https://github.com/igniterealtime/Smack/blob/master/documentation/roster.md

        Roster roster = Roster.getInstanceFor(connection);

	roster.addRosterListener(new RosterListener() {
		public void entriesAdded(Collection<Jid> addresses) { /* handle new entries */ }
		public void entriesDeleted(Collection<Jid> addresses) { /* handle deleted entries */ }
		public void entriesUpdated(Collection<Jid> addresses) { /* handle updated entries */ }
		public void presenceChanged(Presence presence) { /* handle presence change */
		    System.out.println("Presence changed: " + presence.getFrom() + " " + presence);
		}
	    });
    }

    // TO BE TESTED
    private void createMUC(XMPPConnection connection, final EntityBareJid jid, final String roomName)
	throws org.jxmpp.stringprep.XmppStringprepException,
	       org.jivesoftware.smack.SmackException.NoResponseException,
	       org.jivesoftware.smack.XMPPException.XMPPErrorException,
	       org.jivesoftware.smack.SmackException.NotConnectedException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.MucAlreadyJoinedException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.MissingMucCreationAcknowledgeException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.NotAMucServiceException,
	       java.lang.InterruptedException {
	// example from https://www.baeldung.com/xmpp-smack-chat-client
	MultiUserChatManager manager = MultiUserChatManager.getInstanceFor(connection);
	MultiUserChat muc = manager.getMultiUserChat(jid);
	Resourcepart room = Resourcepart.from(roomName);
	muc.create(room).makeInstant();
    }

    // TO BE TESTED
    private void createReservedRoom(XMPPConnection connection, final EntityBareJid jid, final String roomName, final String [] owners)
	throws org.jxmpp.stringprep.XmppStringprepException,
	       org.jivesoftware.smack.XMPPException.XMPPErrorException,
	       org.jivesoftware.smack.SmackException.NotConnectedException,
	       org.jivesoftware.smack.SmackException.NoResponseException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.MucAlreadyJoinedException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.MissingMucCreationAcknowledgeException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.MucConfigurationNotSupportedException,
	       org.jivesoftware.smackx.muc.MultiUserChatException.NotAMucServiceException,
	       java.lang.InterruptedException {
	// example from https://www.baeldung.com/xmpp-smack-chat-client
	// owners can be: String[] owners = new String[] { "baeldung@jabb3r.org", "baeldung2@jabb3r.org" };
	Set<Jid> ownersJid = JidUtil.jidSetFrom(owners);

	MultiUserChatManager manager = MultiUserChatManager.getInstanceFor(connection);
	MultiUserChat muc = manager.getMultiUserChat(jid);
	Resourcepart room = Resourcepart.from(roomName);
	muc.create(room)
	    .getConfigFormManager()
	    .setRoomOwners(ownersJid)
	    .submitConfigurationForm();
    }
    
    public static void main(String[] args) throws Exception
    {
	final String appname = "java " + HelloSmack.class.getName();
        if (args.length < 1) {
	    System.err.println("usage: " + appname + " <xmpp-domain> [<hostname>]");
	    System.exit(1);
	}

	String xmppDomain = args[0];
	String hostname = xmppDomain;
	
	if (args.length == 2) {
	    hostname = args[1];
	}

	String version = SmackConfiguration.getVersion();
	SmackConfiguration.DEBUG = true;
	System.out.println("Smack version " + version);
        System.out.println("hostname: " + hostname);
	String user = BasicInput.askFor("user: ");
	String password = BasicInput.askForPassword("password: ");

        // disabled, ifpossible, required
        ConnectionConfiguration.SecurityMode securityMode = ConnectionConfiguration.SecurityMode.disabled;
        String portSetting = "5222"; // in the real project, will be defined by an option or a configuration setting
        int port = Integer.parseInt(portSetting);
	
	XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
	    .setUsernameAndPassword(user, password)
	    .setXmppDomain(xmppDomain)
	    .setHost(hostname)
            .setSecurityMode(securityMode)
            .setPort(port)
	    .build();
	
	AbstractXMPPConnection connection = new XMPPTCPConnection(config);

	if (connection == null) {
	    System.err.println("Failed to instantiate a connection");
	    System.exit(1);
	}
	
	connection.connect();
	System.out.println("Connected to " + hostname);
	connection.login();
	System.out.println("Logged as " + user);

        ChatManager chatManager = ChatManager.getInstanceFor(connection);
        EntityBareJid jid = JidCreate.entityBareFrom(user);
        Chat chat = chatManager.chatWith(jid);

        chatManager.addIncomingListener(new IncomingChatMessageListener() {
		@Override
		public void newIncomingMessage(EntityBareJid from, Message message, Chat chat) {
		    System.out.println("New message from " + from + ": " + message.getBody());
		}
	    });
	
	chat.send("Hello from " + user + "!");

        //Presence.Type presenceType = Presence.Type.unavailable;
        //String presenceMessage = "Gone fishing";
        //setPresence(connection, Presence.Type.unavailable, "Gone fishing");
        connection.disconnect();
    }
}
